/*
 *
 *  Created on: Mar 1, 2021
 *      Author: astrisw
 */

#pragma once

#include <Base_Provider.h>
#include <unordered_map>
#include <librdkafka/rdkafkacpp.h>
#include <atomic>
#include <thread>
#include <Delivery_Report.h>
#include <Base_Configurator.h>

namespace inaf::oasbo::Providers {
class KafkaProvider: public BaseProvider {
protected:

	RdKafka::Producer *producer = nullptr;
	std::atomic<bool> stopFlag = false;
	std::thread pollThread;
	DeliveryReportCb *dr_cb = nullptr;

	void pollingThreadFunction();

	KafkaProvider();
	KafkaProvider(std::string ip, int port, std::string topic);

public:

	std::string brokerIp;
	int brokerPort;

	void setDest(std::string dest) override {
		this->dest = dest;
	}
	std::string getDest() override {
		return dest;
	}

	int write(PacketLib::BasePacket&) override;
	int write(PacketLib::BasePacket&, std::string dest) override;

	int close() override;
	int open() override;
	bool isOpen() override;
	~KafkaProvider();

	friend class Builder;
	class Builder {
	protected:
		KafkaProvider *provider;

	public:

		std::string ip_key { "ip" };
		std::string port_key { "port" };
		std::string topic_key { "topic" };

		Builder();
		Builder(std::string ip, int port, std::string topic);
		~Builder();

		void reset();

		Builder* configFrom(Configurators::BaseConfigurator *conf);

		Builder* setIp(std::string ip);

		Builder* setPort(int port);

		Builder* setTopic(std::string topic);

		KafkaProvider* getProvider();
	};
};
}
